package drakcluster::pxe_conf;

use strict;

use pxe_server_cluster;
use cluster_serverconf;
use common;
use ugtk2 qw(:wrappers :create);


sub read() {
    { pxe_label => pxe_server_cluster::getDefaultLabel() };
}

sub done_pxe {
    my ($conf) = @_;
    pxe_server_cluster::setDefaultLabel($conf->{pxe_label});
}

sub done_spe {
  my ($confspe) = @_;
  pxe_server_cluster::setDefaultSpecial($confspe->{special_label});
}

sub done_eth {
  my ($confeth) = @_;
  pxe_server_cluster::change_interface($confeth->{eth});
}

sub create {
    my ($conf, $confspe, $confeth) = @_;

    my $apply;
    my $combo = Gtk2::Combo->new;
    $combo->set_use_arrows_always(1);
    $combo->disable_activate;
    $combo->set_popdown_strings(@{ pxe_server_cluster::get_pxe_labels() });
    $combo->entry->set_text($conf->{pxe_label});
    my $ignore;
    $combo->entry->signal_connect(changed => sub {
				    return if $ignore;
				    $apply->set_sensitive(1);
				    $conf->{pxe_label} = $combo->entry->get_text;
				  });

    my $applyspe;
    my $combospe = Gtk2::Combo->new;
    $combospe->set_use_arrows_always(1);
    $combospe->disable_activate;
    $combospe->set_popdown_strings(@{ pxe_server_cluster::get_special_labels() });
    $combospe->entry->set_text($confspe->{special_label});
    my $ignorespe;
    $combospe->entry->signal_connect(changed => sub {
				    return if $ignorespe;
				    $applyspe->set_sensitive(1);
				    $confspe->{special_label} = $combospe->entry->get_text;
				  });

    my $applyeth;
    my $comboeth = Gtk2::Combo->new;
    $comboeth->set_use_arrows_always(1);
    $comboeth->disable_activate;
    $comboeth->set_popdown_strings(@{ pxe_server_cluster::get_eth() });
    $comboeth->entry->set_text($confeth->{eth});
    my $ignoreeth;
    $comboeth->entry->signal_connect(changed => sub {
				    return if $ignoreeth;
				    $applyeth->set_sensitive(1);
				    $confeth->{eth} = $comboeth->entry->get_text;
				  });


    my $help_txt = N("
Select your default PXE entry
local :\t\tLocal boot (for booting on the local disks)
linux :\t\tInstall a golden node
freedos :\t\tFreedos boot
auto    :\t\tAutomated installation a golden node
ka :\t\t\tBoot for duplication process");

    my $help_txtspe = N("
Select the boot options for PXE entries
acpi :\t\t ACPI support is enabled
noacpi :\t\t ACPI support is disabled
apic :\t\t\t APIC support is enabled
noapic :\t\t APIC support is disabled
noall :\t\t APIC and ACPI are enabled (default)
all :\t\t APIC and ACPI are disabled");

    my $help_txteth =  N("
You can choose on which ethernet interface your node will make its DHCP request");

    $apply = Gtk2::Button->new(N("Set default PXE boot image"));
    $apply->set_sensitive(0);
    $apply->signal_connect(clicked => sub {
			     $apply->set_sensitive(0);
			     done_pxe($conf);
			   });

    $applyspe = Gtk2::Button->new(N("Set boot options"));
    $applyspe->set_sensitive(0);
    $applyspe->signal_connect(clicked => sub {
                             $applyspe->set_sensitive(0);
                             done_spe($confspe);
                           });

    $applyeth = Gtk2::Button->new(N("Select this ethernet interface as default NIC"));
    $applyeth->set_sensitive(0);
    $applyeth->signal_connect(clicked => sub {
                             $applyspe->set_sensitive(0);
                             done_eth($confeth);
                           });

    my $rebuild;
    $rebuild = Gtk2::Button->new(N("Rebuild PXE images (rebuild all images and reset default boot options)"));
    $rebuild->set_sensitive(1);
    $rebuild->signal_connect(clicked => sub {
			       $::in->ask_yesorno('', N("Are you sure you want to rebuild all your PXE images ?\n\nAll special boot disk options will be destroy. \n\nIt uses /var/install/cluster/images/*.img to regenerate all images")) or return;
				 pxe_server_cluster::rebuild_img();
			     });

    my $sd = Gtk2::CheckButton->new("Export installation's display to the server");
    $sd->set_sensitive(1);
    if (get_display() eq "1") { $sd->set_active(1) } else { $sd->set_active(0) }
    $sd->signal_connect(clicked => sub {
			  my $s = $sd->get_active;
			  if ($s =~ /1/) { display_change("A")
					 } else { display_change("D") }
			});

    gtkpack_(Gtk2::VBox->new(0, 0),
	     1, $help_txt,
	     1, gtkpack_(Gtk2::HBox->new, 0, N("PXE Entry"), 1, $combo),
	     0, $apply,
	     1, Gtk2::HSeparator->new,
	     1, $help_txteth,
	     1, gtkpack_(Gtk2::HBox->new, 0, N("Ethernet Device"), 1, $comboeth),
	     0, $applyeth,
	     1, Gtk2::HSeparator->new,
#	     1, $sd,
#	     1, Gtk2::HSeparator->new,
	     0, $help_txtspe,
	     1, gtkpack_(Gtk2::HBox->new, 0, N("Boot disk option"), 1, $combospe),
	     0, $applyspe,
	     1, Gtk2::HSeparator->new,
	     0, $rebuild),
	       sub {
		 $apply->get_sensitive and return; # modified but not applied, keeping modification
		 put_in_hash($conf, &read);
		 $ignore = 1;
		 $combo->entry->set_text($conf->{pxe_label});
		 $ignore = 0;
	       };
}

1;
