package interface_cluster;

# Author : Daniel Viard <dviard@mandrakesoft.com>
# Version 0.1
# please report bug to: clic-public-dev@mandrakesoft.com

use strict;
use MDK::Common;
use cluster_fonction_common;
use cluster_serverconf;
use cluster_set_admin;
use cluster_set_compute;
use server_cluster;
use maui_cluster;


sub get_node_list() {
  my %nodes;
  foreach my $line (cat_(cluster_serverconf::cluster_data()->{NODESFILEADMIN})) {
    my $domainname = cluster_serverconf::dns_data()->{DOMAINNAME};
    my $comp_domainname = cluster_serverconf::dns_data()->{DOMAINCOMP};
    if (my ($name, $active, $group) = $line =~ /(\S+)\.$domainname:(\w):(\S+)\n/) {
      my $compnode = get_compute_name($name);
      my ($cpu, $activec, $partition) = cat_(cluster_serverconf::cluster_data()->{NODESFILE}) =~ /$compnode\.$comp_domainname:(\d):(\w):(\S*):\d+:\d+/;
      $nodes{$name} = { name => $name, cpu => $cpu, compute => $activec eq 'A' ? 1 : 0, admin => $active eq 'A' ? 1 : 0, partition => $partition, group => $group };
    }
  }
  \%nodes;
}

sub get_number_of_nodes() {
    my $nodes = get_node_list();
    int(keys %$nodes);
}

sub get_partition_list() {
  maui_cluster::partitions_list();
}

sub get_group_list() {
  my $urpmi_conf = cluster_serverconf::urpmi_data()->{URPMICFG};

  my @group = cat_($urpmi_conf) =~ /(\S+):\S+:/g;
  [ uniq(@group) ];
}

sub get_users_list() {
  my %users = (
      map {
	  $_ => { map { $_ => 1 } @{maui_cluster::partitions_of_user($_)} };
#      } @{maui_cluster::get_maui_users()}
      } @{nis_cluster::get_nis_users()}
  );
  \%users;
}

our @ISA = qw(Exporter);
our @EXPORT = qw(get_loadavg);

sub get_loadavg {
  my ($node) = @_;
  # node3.guibland.com     1 (    0/   59) [  0.03,  0.13,  0.04] [   0.0,   0.0,   0.1,  97.9] OFF
  my @cmd = `gstat -a -l -1`;
  foreach (grep { /$node/ } @cmd) {
    (my ($load1, $load5, $load15, $u, $s, $idle) = /$node\s+\d\s\(.*\)\s\[\s+(\d+\.\d+),\s+(\d+\.\d+),\s+(\d+\.\d+)\]\s+\[\s+(\d+\.\d+),\s+\d+\.\d+,\s+(\d+\.\d+),\s+(\d+\.\d+).*\]/);
    my $all = "[ $load1 | $load5 | $load15 ]  [ $u | $s | $idle ]";
    return $all;
  }
}


sub get_states {
    my ($nodes) = @_;

    my $domainname = cluster_serverconf::dns_data()->{DOMAINNAME};
    my $line = `gstat -a -l`;
    map {
      $_ => $line =~ /$_\.$domainname/ ? 1 : -1;
    } keys %$nodes; # -1 is down, 0 is unknown, 1 is up
}

sub chg_node_active_admin {
  my ($node, $new_active) = @_;
  my $domainname = cluster_serverconf::dns_data()->{DOMAINNAME};
  $new_active = $new_active ? 'A' : 'D';
  substInFile {
    s/($node\.$domainname:)\w(:\S+\n)/$1$new_active$2/g;
  } cluster_serverconf::cluster_data()->{NODESFILEADMIN};
  cluster_set_admin::drak_ar_admin();
  urpmipp_ar_change("$node.$domainname");
}

sub chg_node_active_compute {
  my ($node, $new_active) = @_;
  $node = get_compute_name($node);
  $new_active = $new_active ? 'A' : 'D';
  my $comp_domainname = cluster_serverconf::dns_data()->{DOMAINCOMP};
  substInFile {
    s/($node\.$comp_domainname:\d):\w:(.*)/$1:$new_active:$2/g;
  } cluster_serverconf::cluster_data()->{NODESFILE};
  cluster_set_compute::drak_ar_compute();
}

sub chg_node_part {
    my ($node, $new_part) = @_;
    $node = get_compute_name($node);
    my $comp_domainname = cluster_serverconf::dns_data()->{DOMAINCOMP};
    substInFile {
	s/($node\.$comp_domainname:\d:\w:)\S*(:\d+:\d+\n)/$1$new_part$2/g;
    } cluster_serverconf::cluster_data()->{NODESFILE};
    change_partition_of_node("$node.$comp_domainname", $new_part);
}

sub chg_user_part {
  my ($partition, $user, $new_val) = @_;

  if ($new_val == 1) {
    maui_cluster::add_user_in_partition($user, $partition);
  } else {
    maui_cluster::remove_user_from_partition($user, $partition);
  }
}

sub cmd_to_exec {
  my ($mode, $selection, $verbose, $cmd) = @_;

  $verbose = $verbose == 1 ? ' -v' : '';

  my $domainname = cluster_serverconf::dns_data()->{DOMAINNAME};

  my $gexec_svrs = $ENV{GEXEC_SVRS};
  my $node_list = join(" ", map { "$_->{name}.$domainname" } @$selection);
  my $pt = "/usr/bin";
  $mode eq 'gexec' ? "cd /tmp; export GEXEC_SVRS=\" $node_list\"; $pt/${mode}_wrap -n 0$verbose $cmd; export GEXEC_SVRS=\"$gexec_svrs\"" :
    "$pt/${mode}_wrap -c ssh$verbose -m " . join(" -m ", map { $_->{name} } @$selection) . " -- $cmd";
}

sub cmd_poweron {
  my ($node) = @_;
  eval { sys("wakeup_node.pl -n $node") };
}

sub cmd_poweroff {
  my ($nodes) = @_;
  eval { sys("dssh $nodes -f -e halt") };
}

sub cmd_reboot {
  my ($nodes) = @_;
  eval { sys("dssh $nodes -f -e reboot") };
}

sub cmd_install_pkg {
  my ($groups, $pkgs) = @_;

  join(';', map { "urpmi --parallel --auto -a $_ $pkgs " } @$groups);
}

sub cmd_remove_pkg {
  my ($groups, $pkgs) = @_;

  join(';', map { "urpme --parallel --auto $_ $pkgs " } @$groups);
}

sub add_nodes {
    my ($nb, $golden, $dev, $skip) = @_;
    "setup_auto_add_nodes.pl -a -g $golden -n $nb -p $dev $skip";
}

sub remove_nodes {
    my ($nodes) = @_;
    map { remove_node_in_admin($_) } @$nodes;
    map { remove_node_in_compute($_) } @$nodes;
    map { dhcpnode_cluster::remove_node_in_dhcp($_) } @$nodes;
    server_cluster::need_after_ar_node();
#    join(" ; ", map { "setup_auto_remove_nodes.pl -a -d $_.$domainname" } @$nodes);
}

1;
